#ifndef EXTATTR_HELPERS_H
#define EXTATTR_HELPERS_H

void setattr_warn (const char *funcname,
                   const char *attrname,
                   const int the_errno);

#endif /* EXTATTR_HELPERS_H */
